function [oblm, oblt, eqeq, dpsi, deps] = etilt1 (tjd)

% this function computes quantities related to the orientation
% of the earth's rotation axis at julian date tjd

% nutation parameters from nut2000a

% input

%  tjd = tdb julian date for orientation parameters

% output

%  oblm = mean obliquity of the ecliptic at date tjd (degrees)
%  oblt = true obliquity of the ecliptic at date tjd (degrees)
%  eqeq = equation of the equinoxes at date tjd (arc seconds)
%  dpsi = nutation in longitude at date tjd (arc seconds)
%  deps = nutation in obliquity at date tjd (arc seconds)

% ported from NOVAS 2.9/3.1

%%%%%%%%%%%%%%%%%%%%%%%%%%%

global tlaste obm obt ee psi eps

t0 = 2451545.0;

seccon = 206264.8062470964;

if (abs(tjd - tlaste) < 1.0e-6)
    
   oblm = obm;
   
   oblt = obt;
   
   eqeq = ee;
   
   dpsi = psi;
   
   deps = eps;
   
   return;
   
end

t = (tjd - t0) / 36525.0;

t2 = t * t;

t3 = t2 * t;

% obtain nutation parameters in seconds of arc

[dpsi, deps] = nut2000a (tjd, 0.0);

psi = dpsi * seccon;

eps = deps * seccon;

% compute mean obliquity of the ecliptic in seconds of arc

obm = 84381.4480 - 46.8150 * t - 0.00059 * t2 + 0.001813 * t3;

% compute true obliquity of the ecliptic in seconds of arc

obt = obm + eps;

% compute equation of the equinoxes in seconds of time

ee = psi / 15.0 * cos (obt/seccon);

% convert obliquity values to degrees

obm = obm / 3600.0;

obt = obt / 3600.0;

tlaste = tjd;

oblm = obm;

oblt = obt;

eqeq = ee;

dpsi = psi;

deps = eps;
